# DoerGPIO Python Library
# Copyright (C) 2024  Pasco Tang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# anyio/arduino/GPIO.py  5/7/2025 Pasco Tang
# An ardunio (serial) based GPIO link

# CONFIGURATION ========================================================

DEBUG = False
# There was the option to use the serial library pre-installed. This has changed so that we know the python lib in use is the one distributed with the doer library.
#If you wish to use the built in one instead change the import doerserial to import serial which is below (Approx line 38)

MIN_PIN = 0
MAX_PIN = 27

IN      = 0
OUT     = 1
PWM     = 2
ADC     = 3
HIGH    = 1
LOW     = 0

PUD_OFF = 20
PUD_DOWN = 21
PUD_UP = 22

VERSION = "DoerGPIO.GPIO 1.1"

# OS INTERFACE =========================================================

from DoerGPIO import protocol
from DoerGPIO import adaptors


#from os import sys, path
#thisdir = path.dirname(path.abspath(__file__))
#sys.path.append(thisdir)

#import doerserial as serial

#Temporarily changing back to normal serial

from DoerGPIO import doerserial

instance = protocol.GPIOClient(adaptors.SerialAdaptor(doerserial.s), DEBUG)

def setup(channel, mode,pull_up_down=None,initial=0):
  if type(channel) is list:
    for c in channel:
      instance.setup(c, mode,pull_up_down,initial)
  else:
    instance.setup(channel, mode,pull_up_down,initial)

def input(channel):
  return instance.input(channel)

def output(channel, value):
  instance.output(channel, value)

def pwm_set_duty(channel, duty):
  instance.pwm_set_duty(channel, duty)

def pwm_set_frequency(channel, frequency):
  instance.pwm_set_frequency(channel, frequency)

def adc_read(channel):
  return instance.adc_read(channel)

def i2c_init(bus=1):
  return instance.i2c_init(bus)

def i2c_scan(bus=1):
  return instance.i2c_scan(bus)

def i2c_write(bus, addr, data):
  return instance.i2c_write(bus, addr, data)

def i2c_read(bus, addr, length):
  return instance.i2c_read(bus, addr, length)

def i2c_write_reg(bus, addr, reg, data):
  return instance.i2c_write_reg(bus, addr, reg, data)

def i2c_read_reg(bus, addr, reg, length):
  return instance.i2c_read_reg(bus, addr, reg, length)

def cleanup(number=0):
  instance.cleanup()


# END



# DoerGPIO Python Library
# Copyright (C) 2024  Pasco Tang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# protocol.py  06/18  Pasco Tang

# Eventually there will be sub modules to the protocol
# to include I2C, SPI, Uart, PWM, code load, and other stuff.
# For now, it only supports the GPIO module and it is loaded
# as the default context
from builtins import bytes
import time
def trace(msg):
  print(msg)

def error(msg):
  trace("error:" + str(msg))

IN  = 0
OUT = 1
PWM = 2
ADC = 3

GPIO_MODE_INPUT  = "I"
GPIO_MODE_OUTPUT = "O"
GPIO_MODE_PWM = "P"
GPIO_MODE_ADC = "A"

GPIO_READ       = "?"
GPIO_ANALOG       = "A"
GPIO_ADC_READ   = "R"
GPIO_VALUE_HIGH = "1"
GPIO_VALUE_LOW  = "0"

PUD_DOWN = 21
PUD_UP = 22
PUD_OFF = 20
GPIO_PULL_DOWN  = "D"
GPIO_PULL_UP  = "U"
GPIO_PULL_NONE = "N"

def _pinch(channel):
  return chr(channel+ord('a'))

def _valuech(value):
  if value == None or value == 0 or value == False:
    return GPIO_VALUE_LOW
  return GPIO_VALUE_HIGH

def _modech(mode):
  if mode == None or mode == IN:
    return GPIO_MODE_INPUT
  return GPIO_MODE_OUTPUT

def _pudch(pud):
  if pud == PUD_DOWN:
    return GPIO_PULL_DOWN
  elif pud == PUD_UP:
    return GPIO_PULL_UP
  else:
    return GPIO_PULL_NONE

def _parse_valuech(ch):
  if ch == GPIO_VALUE_LOW:
    return False
  if ch == GPIO_VALUE_HIGH:
    return True
  error("Unknown value ch:" + str(ch))
  return GPIO_VALUE_HIGH

def _parse_avalue(ch):
  self.trace(ch)
  if ch == GPIO_VALUE_LOW:
    return False
  if ch == GPIO_VALUE_HIGH:
    return True
  error("Unknown value ch:" + str(ch))
  return GPIO_VALUE_HIGH


# CLIENT ===============================================================

class GPIOClient:
  """ The GPIO command set
      Assumes the wire protocol is already in the GPIO mode.
      As we only support the GPIO module at the moment,
      that's a simple assumption to make.
  """
  IN = 0
  OUT = 1
  PWM = 2
  ADC = 3
  DEBUG = False
  PUD_DOWN = 21
  PUD_UP = 22
  BOARDMODE = 0


  def trace(self, msg):
    if self.DEBUG:
      trace(msg)

  def __init__(self, wire, debug=False):
    self.wire = wire
    self.DEBUG = debug


  def setup(self, channel, mode, pull_up_down=None, initial=0):
    #TODO outer wrapper needs to do validation
    #if channel < self.MIN_PIN or channel > self.MAX_PIN:
    #  raise ValueError("Invalid pin")

    pinch = _pinch(channel)
    self.trace(pinch)

    # Check if PWM mode is requested on non-PWM capable pin
    if mode == PWM:
      pwm_pins = [4, 9, 10, 16, 18, 20, 22, 26]
      if channel not in pwm_pins:
        raise ValueError("PWM not support on this pin")
      self._write(pinch + GPIO_MODE_PWM + 'Z')
      return

    # Check if ADC mode is requested on non-ADC capable pin
    if mode == ADC:
      adc_pins = [0, 9, 10, 11, 14, 15, 18, 26]
      if channel not in adc_pins:
        raise ValueError("ADC not support on this pin")
      self._write(pinch + GPIO_MODE_ADC + 'Z')
      return

    modech = _modech(mode)
    
    # For input mode, default to pull up if not specified
    if mode == IN:
      if pull_up_down is None:
        pull_up_down = PUD_UP
      pudch = _pudch(pull_up_down)
      self._write(pinch + modech + pinch + pudch + 'Z')
    # For output mode, don't set pull up/down but set initial value
    elif mode == OUT:
      # Set initial value (defaults to LOW/0)
      self._write(pinch + modech + pinch + _valuech(initial) + 'Z')


  def input(self, channel):
    pinch = _pinch(channel)
    time.sleep(0.01)
    response = self._write(pinch + GPIO_READ + 'Z')
    
    # Parse response in format "GPIO18: 1"
    try:
        response_str = response.decode(encoding='UTF-8').strip()
        gpio_num = int(response_str.split(':')[0].replace('GPIO', ''))
        value = int(response_str.split(':')[1].strip())
        
        # Verify GPIO number matches requested channel
        if gpio_num == channel:
            return value
        else:
            error(f"GPIO number mismatch: expected {channel}, got {gpio_num}")
            return False
    except (ValueError, IndexError) as e:
        error(f"Failed to parse response: {response_str}")
        return False

  def adc_read(self, channel):
    """Read ADC value from specified channel"""
    # Check if pin supports ADC
    adc_pins = [0, 9, 10, 11, 14, 15, 18, 26]
    if channel not in adc_pins:
      raise ValueError("ADC not support on this pin")
    
    pinch = _pinch(channel)
    time.sleep(0.01)
    response = self._write(pinch + GPIO_ADC_READ + 'Z')
    
    # Parse response in format "GPIO0: ADC 2048"
    try:
        response_str = response.decode(encoding='UTF-8').strip()
        gpio_num = int(response_str.split(':')[0].replace('GPIO', ''))
        adc_value = int(response_str.split('ADC')[1].strip())
        
        # Verify GPIO number matches requested channel
        if gpio_num == channel:
            return adc_value
        else:
            error(f"GPIO number mismatch: expected {channel}, got {gpio_num}")
            return 0
    except (ValueError, IndexError) as e:
        error(f"Failed to parse ADC response: {response_str}")
        return 0

  def output(self, channel, value):
    #TODO outer wrapper needs to do validation
    #if channel < self.MIN_PIN or channel > self.MAX_PIN:
    #  raise ValueError("Invalid pin")
    ch = _pinch(channel)
    v = _valuech(value)

    time.sleep(0.01)
    if value == None or value == 0 or value == False:
      self._write(ch + GPIO_VALUE_LOW + 'Z')
    else:
      self._write(ch + GPIO_VALUE_HIGH + 'Z')
    #TODO read and verify echoback

  def cleanup(self):
    pass

  def pwm_set_duty(self, channel, duty):
    """Set PWM duty cycle on specified channel"""
    # Check if pin supports PWM
    pwm_pins = [4, 9, 10, 16, 18, 20, 22, 26]
    if channel not in pwm_pins:
      raise ValueError("PWM not support on this pin")
    
    pinch = _pinch(channel)
    time.sleep(0.01)
    self._write(pinch + "C" + str(duty) + 'Z')

  def pwm_set_frequency(self, channel, frequency):
    """Set PWM frequency on specified channel"""
    # Check if pin supports PWM
    pwm_pins = [4, 9, 10, 16, 18, 20, 22, 26]
    if channel not in pwm_pins:
      raise ValueError("PWM not support on this pin")
    
    pinch = _pinch(channel)
    time.sleep(0.01)
    self._write(pinch + "F" + str(frequency) + 'Z')

  def i2c_init(self, bus=1):
    cmd = f'@{bus}INZ'
    response = self._write(cmd)
    try:
      resp_str = response.decode('UTF-8').strip()
      if 'ERR' in resp_str:
        return False
      return 'OK INIT' in resp_str
    except Exception as e:
      error(f"i2c_init failed: {e}")
      return False

  def i2c_scan(self, bus=1):
    cmd = f'@{bus}SCZ'
    response = self._write(cmd)
    try:
      resp_str = response.decode('UTF-8').strip()
      if 'ERR' in resp_str:
        return []
      payload = resp_str.split('SCAN')[1].strip()
      if payload == 'NONE' or payload == '':
        return []
      return [int(x, 16) for x in payload.split()]
    except Exception as e:
      error(f"i2c_scan failed: {e}")
      return []

  def i2c_write(self, bus, addr, data):
    data = bytes(data)
    addr_hex = f'{addr:02X}'
    data_hex = data.hex().upper()
    cmd = f'@{bus}WR{addr_hex},{data_hex}Z'
    response = self._write(cmd)
    try:
      resp_str = response.decode('UTF-8').strip()
      if 'ERR' in resp_str:
        raise RuntimeError(f"I2C write error: {resp_str}")
      return True
    except RuntimeError:
      raise
    except Exception as e:
      error(f"i2c_write failed: {e}")
      return False

  def i2c_read(self, bus, addr, length):
    addr_hex = f'{addr:02X}'
    len_hex = f'{length:02X}'
    cmd = f'@{bus}RD{addr_hex},{len_hex}Z'
    response = self._write(cmd)
    try:
      resp_str = response.decode('UTF-8').strip()
      if 'ERR' in resp_str:
        raise RuntimeError(f"I2C read error: {resp_str}")
      payload = resp_str.split('RX')[1].strip()
      if payload == '':
        return b''
      return bytes(int(x, 16) for x in payload.split())
    except RuntimeError:
      raise
    except Exception as e:
      error(f"i2c_read failed: {e}")
      return b''

  def i2c_write_reg(self, bus, addr, reg, data):
    data = bytes(data)
    addr_hex = f'{addr:02X}'
    reg_hex = f'{reg:02X}'
    data_hex = data.hex().upper()
    cmd = f'@{bus}MW{addr_hex},{reg_hex},{data_hex}Z'
    response = self._write(cmd)
    try:
      resp_str = response.decode('UTF-8').strip()
      if 'ERR' in resp_str:
        raise RuntimeError(f"I2C write_reg error: {resp_str}")
      return True
    except RuntimeError:
      raise
    except Exception as e:
      error(f"i2c_write_reg failed: {e}")
      return False

  def i2c_read_reg(self, bus, addr, reg, length):
    addr_hex = f'{addr:02X}'
    reg_hex = f'{reg:02X}'
    len_hex = f'{length:02X}'
    cmd = f'@{bus}MR{addr_hex},{reg_hex},{len_hex}Z'
    response = self._write(cmd)
    try:
      resp_str = response.decode('UTF-8').strip()
      if 'ERR' in resp_str:
        raise RuntimeError(f"I2C read_reg error: {resp_str}")
      payload = resp_str.split('RX')[1].strip()
      if payload == '':
        return b''
      return bytes(int(x, 16) for x in payload.split())
    except RuntimeError:
      raise
    except Exception as e:
      error(f"i2c_read_reg failed: {e}")
      return b''


  # redirector to wrapped comms link
  def _open(self, *args, **kwargs):
    self.trace("open")
    self.wire.open(*args, **kwargs)

  def _write(self, *args, **kwargs):
    self.trace("write:" + str(*args) + " " + str(**kwargs))
    self.wire.write(*args, **kwargs)
    time.sleep(0.01)
    v=self._read(100, termset="\n")
    self.trace("read back:" + v.decode(encoding='UTF-8'))
    return v

  def _read(self, *args, **kwargs):
    self.trace("read")
    return self.wire.read(*args, **kwargs)

  def _close(self):
    self.trace("close")
    self.wire.close()



# DoerGPIO Python Library
# Copyright (C) 2024  Pasco Tang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# ports_win32.py  06/18/2025  Pasco Tang
#
# Scan the windows registry to find a list of COM ports

try:
  import _winreg as registry # python2
except ImportError:
  import winreg as registry # python3

KEY = r"HARDWARE\DEVICEMAP\SERIALCOMM"

def scan():
  ports = []

  reg = registry.ConnectRegistry(None, registry.HKEY_LOCAL_MACHINE)
  try:
    key = registry.OpenKey(reg, KEY)
  except:
    # If there is no SERIALCOMM registry entry
    # it means this computer has never seen a serial port.
    # Best action is to return an empty ports list. 
    # When the device is inserted, windows will create the entry for us.
    return ports

  i = 0
  while True:
    try:
      value = registry.EnumValue(key, i)   
      name, value, vtype = value
      print("port[" + str(i) + "]:" + str(value))
      ports.append(value)
      i += 1
    
    except EnvironmentError:
      break
    
  return ports


# TESTER

if __name__ == "__main__":
  d = find()
  print(str(d))
  
# END



# DoerGPIO Python Library
# Copyright (C) 2024  Pasco Tang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# ports_unix.py  06/18/2025  Pasco Tang
#
# Get a list of ports on a unix system
# Note that the precise /dev/* filter depends on the platform

# SYSTEM AND VERSION VARIANCE ==========================================

#this is for linux
DEV_TTY    = "/dev/tty*"

#TODO for mac, it's /dev/cua*???
  
  
# BODY =================================================================

import glob

def scan():
  """ scan devices that might be com ports """
  devices = glob.glob(DEV_TTY)
  #print("found " + str(len(devices)) + " devices")
  return devices


# TEST HARNESS =========================================================
 
if __name__ == "__main__":
  d = scan()
  print(str(d)) 
    
# END



# DoerGPIO Python Library
# Copyright (C) 2024  Pasco Tang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import serial
import DoerGPIO.portscan as portscan

DEBUG = True
# STATIC REDIRECTORS ===================================================

# Find out if there is a pre-cached port name.
# If not, try and find a port by using the portscanner
#
#name = portscan.getName()
#if name != None:
#  if DEBUG:
#    print("Using port:" + name)
#  PORT = name
#else:
#  name = portscan.find()
#  if name == None:
#    raise ValueError("No port selected, giving in")
#  PORT = name
#  print("Your anyio board has been detected")
#  print("Now running your program...")

doerGPIOPort = ""
import serial.tools.list_ports
serialPorts = serial.tools.list_ports.comports()

for port in serialPorts:
    if( "1a86:7523" in port[2].lower() or "1a86:7523" in port[2].lower()):
        if(DEBUG):
            print ("Doer GPIO board Found on port: "+port[0])
        doerGPIOPort = port[0]
        break
if(doerGPIOPort == ""):
	print ("\nError: Doer GPIO board is not detected.\nFor more support please visit our website at https://doer.ee\nPress enter to close.")
	input()
	exit()

#print ("Debug cable fully detected fine.\n Press enter to launch the terminal.")


BAUD = 230400


s = serial.Serial(doerGPIOPort)
s.baudrate = BAUD
s.parity   = serial.PARITY_NONE
s.databits = serial.EIGHTBITS
s.stopbits = serial.STOPBITS_ONE
#s.write_timeout = 
timeout = None

s.close()
s.port = doerGPIOPort
s.open()

# Send version command and read response with timeout
s.timeout = 2  # Set timeout to 2 seconds
while True:
    try:
        s.write(b'VVZ')
        response = s.readline().decode('utf-8').strip()
        if response:
            # Check if response matches DoerGPIO_HWX.Y_SWZ.W format
            if response.startswith('DoerGPIO_HW'):
                try:
                    # Split into hardware and software parts
                    hw_part, sw_part = response.split('_SW')
                    hw_version = hw_part[11:]  # Remove 'DoerGPIO_HW' prefix
                    sw_version = sw_part
                    
                    # Verify both versions are in X.Y format
                    hw_major, hw_minor = hw_version.split('.')
                    sw_major, sw_minor = sw_version.split('.')
                    
                    if (hw_major.isdigit() and hw_minor.isdigit() and 
                        sw_major.isdigit() and sw_minor.isdigit()):
                        print(f"Hardware Version: {hw_version}")
                        print(f"Software Version: {sw_version}")
                        break
                    else:
                        print("Wrong Board Version, contact customer support at cs@doer.ee")
                        s.close()
                        exit()
                except ValueError:
                    print("Wrong Board Version, contact customer support at cs@doer.ee")
                    s.close()
                    exit()
            else:
                print("Wrong Board Version, contact customer support at cs@doer.ee")
                s.close()
                exit()
        else:
            print("Connection failed. Retrying...")
        retry = input("Press Enter to retry or 'q' to quit: ")
        if retry.lower() == 'q':
            print("Exiting...")
            s.close()
            exit()
    except serial.SerialTimeoutException:
        print("Connection failed. Retrying...")
        retry = input("Press Enter to retry or 'q' to quit: ")
        if retry.lower() == 'q':
            print("Exiting...")
            s.close()
            exit()



# DoerGPIO Python Library
# Copyright (C) 2024  Pasco Tang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# adaptors.py  06/18/2025  Pasco Tang
#
# Some adaptors to make the inner connectivity classes all work
# in a standard way
from pprint import pprint
import time

# adapt a pyserial (or anything else) to our common interface
class SerialAdaptor:

  def __init__(self, serial):
    self.serial = serial

  def open(self, *args):
    self.serial.open(*args)

  def close(self, *args):
    self.serial.close(*args)

  # read data between minsize and maxsize chars.
  # optional terminator set to mark end of packet.
  # timeout if any one read takes longer than timeout period.
  # i.e. whole packet may take longer to come in

  def read(self, maxsize=1, minsize=None, termset=None, timeout=None):
    if minsize == None:
      minsize = maxsize

    remaining = maxsize
    if termset != None:
      readsz = 1
    else:
      readsz = remaining

    buf = b''
    # print("Enter read")
    while len(buf) < minsize:
      data = self.serial.read(readsz)
      if (len(data) == 0):
        time.sleep(0.1) # prevent CPU hogging
      else:
        # print("just read:" + data.decode('utf-8'))
        buf = buf + data
        remaining -= len(data)
        if termset != None:
          if data[0] in termset.encode('ascii'):
            break # terminator seen

    return buf

  def write(self, str):
    self.serial.write(str.encode('ascii'))



from setuptools import setup, find_packages

setup(
    name="DoerGPIO",
    version="1.1",
    packages=find_packages(),
    install_requires=[
        "pyserial",  # Required for serial communication
    ],
    author="Pasco Tang",
    author_email="pasco@doer.ee",  # You should add your email here
    description="USB GPIO interface for Python",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://doer.ee",  # You should add your repository URL here
    license="GPL-3.0-or-later",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.9",
) 



# DoerGPIO Python Library
# Copyright (C) 2024  Pasco Tang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# portscan.py  06/18/2025  Pasco Tang
#
# Prompt user and find a newly inserted com port
# Appropriately configured this should work on mac an linux

import time

# CONFIGURATION ========================================================

# The file that will be created to cache the found port name
CACHE_NAME = "portscan.cache"

# The time, in seconds, to wait for any drivers to unload.
# When the user presses ENTER after the "remove device" message,
# it sometimes takes some computers a little bit of time to safely
# unload the driver. This prevents false readings.
DRIVER_UNLOAD_TIME = 2

# The time, in seconds, to wait for any drivers to load.
# When the user presses ENTER after the "insert device" message,
# it sometimes takes some computers a little bit of time to safely
# load the driver. This prevents false readings.

DRIVER_LOAD_TIME = 4

def message(msg):
  print(msg)
  
def nl():
  print("\n")


# SYSTEM AND VERSION VARIANCE ==========================================

import sys
import os

if sys.version_info.major >= 3:
  ask = input
else:
  ask = raw_input
    
if os.name == 'nt': #sys.platform == 'win32':
    import DoerGPIO.ports_win32 as ports
elif os.name == 'posix':
    import DoerGPIO.ports_unix as ports
else:
    raise ImportError("No port lister available for:" + os.name)


# HELPERS ==============================================================

def getYesNo(msg):
  """ Ask user for yes/no and return a boolean """
  answer = ask(msg + " (Y/N)")
  answer = answer.upper()
  if answer in ['YES', 'Y']:
    return True
  return False
  
def getAdded(before, after):
  """ Find any items in 'after' that are not in 'before' """

  #message("before:" + str(before))
  #message("after:" + str(after))
  
  # Any items in 'before' are removed from 'after'
  # The remaining list is of the newly added items
  
  for b in before:
    try:
      i = after.index(b)
      after.remove(b)
    except ValueError:
      pass

  #message("new:" + str(after))
  return after  
  
  
# BODY =================================================================

def scan():
  """ scan devices repeatedly until new one found, or user gives in """
  message("Scanning for serial ports")
  while True:
    # prompt to remove device
    ask("remove device, then press ENTER")

    message("scanning...")
    time.sleep(DRIVER_UNLOAD_TIME) # to allow driver to unload
    before = ports.scan()
    beforec = len(before)
    message("found " + str(beforec) + " devices")

    # prompt to insert device
    ask("plug in device, then press ENTER")

    message("scanning...")
    time.sleep(DRIVER_LOAD_TIME) # to allow driver to load
    after = ports.scan()
    afterc = len(after)
    message("found " + str(afterc) + " devices")

    # diff the lists
    added = getAdded(before, after)
    
    if len(added) == 0:
      # No new ones, try again?
      message("no new devices detected")
      yes = getYesNo("Try again?")
      if yes:
        continue
      return None
    
    elif len(added) > 1:
      # Show a menu and get a choice
      while True:
        message("more than one new device found")
        for i in range(len(added)):
          message(str(i+1) + ":" + added[i])
        a = ask("which device do you want to try?")
        try:
          a = int(a)
          if a < 1 or a > len(added):
            message("out of range, try again")
            continue
          a -= 1
          return added[a]
        except:
          pass

    else: 
      # only 1 new, select it
      message("found 1 new device")
      dev = added[0]
      message("selected:" + dev)
      return dev
      
    
def remember(device):
  """ Remember this device for next time """
  # prompt if you want it remembered
  yes = getYesNo("Do you want this device to be remembered?")

  if yes:
    # Remember it
    f = open(CACHE_NAME, "w")
    f.write(device + "\n")
    f.close()

def find():
  """ Try to find a newly inserted device, by prompting user """
  dev = scan()
  if dev != None:
    remember(dev)
  return dev
  
def forget():
  """forget the remembered cached name if stored"""
  # Remove any existing cache file
  try:
    os.remove(CACHE_NAME)
  except:
    pass
  
def getName():
  """read the remembered cached named, None if none stored"""
  try:
    f = open(CACHE_NAME, "r")
    name = f.readline().strip()
    f.close()
    return name  
  except IOError:
    message("could not open cache file")
    return None

def main():
  message("*" * 79)
  message("SERIAL PORT SCANNER PROGRAM")
  message("*" * 79)
  
  n = getName()
  if n == None:
    message("No name remembered")
  else:
    message("Already remembered:" + n)
    message("forgetting...")
    forget()
    
  d = find()
  if d == None:
    message("nothing found")
  else:
    message("found device:" + d)

# TESTER  
if __name__ == "__main__":
  main()
  
# END
